from reaper_python import *
from sws_python import *
from contextlib import contextmanager
from random import randint


################
                #
debug = True   #-
                #
################

def msg(m):
    if (debug):
        RPR_ShowConsoleMsg(str(m)+'\n')

    
class RC:
    """
    ReaChorder class.
    """
    
    #this is static data, access it without creating an instance of RC by doing RC.circleFifthsInnerTxt(i) etc
    
    REACHORD="ReaChord1"
    #Default settings values.  Values are directly changed via settings window and saved using Reaper's save/recall functionality
    # values are do/don't create midi and midi channel
    CHORDS="Chords" ; MELODY="Melody" ; BASS="Bass" ; TRANSPOSE="Transpose" ; SCALE="Scale" ; TRIGGER="Trigger"
    REGION="Region"
    TRUE="True"  ;  FALSE="False"
    pSettingsOnOff = 0   ;   pSettingsChannel = 1
    Settings = {  #chans 0 - 15
        CHORDS: ["True",   1],    MELODY: ["True",   2],     BASS: ["False",  3],  REGION: ["False", "False"]
        #,TRANSPOSE: ["False", 15], SCALE:  ["False", 16],    TRIGGER:["False", 10]
    }
    
    
    Layouts = ('Select layout...','Chords only','Melody only','Chords and Melody')
    
    Moods = ('Select mood of the song...',)  #create tuple with one entry
    
    InKey = ('In Key:','A','B','C','D','E','F','G','A#','C#','D#','F#','G#')
    
    Scales = ('Select scale...','Major','Minor')
    
    entriesStructures = ('Select structure...',)
    
    allowedChars = "VC"      #"IVPCBMDSQLXE"
    pStructureChars = 1 #2nd pos
    pStructureName = 0 
    Structures = [
        ['Verse',             'V'         ],
        ['Verse-Chorus',      'VC'        ],
        ['Verse-Verse-Chorus','VVC'       ],    #### we can now add/remove structures here
        ['VC VC',             'VCVC'      ],    #### and it will automagically update
        ['VV CV',             'VVCV'      ],
        ['VV CV CV',          'VVCVCV'    ],
        ['VC VC VV C',        'VCVCVVC'   ],
        ['VC VC V CC',        'VCVCVCC'   ]
    ]
    
    circleFifthsInnerTxt = [
        #      Am               Bm             Cm               Dm               Em              Fm               Gm
        ['F', 'C', 'G' ],['G', 'D','A'],['G#','D#','A#'],['A#','F','C' ],['C', 'G', 'D' ],['C#','G#','D#'],['D#','A#','F']
    ]
    
    circleFifthsOuterTxt = [
        #       A                   B                   C                   D                   E                   F                   G
        ['Bm', 'F#m','C#m'],['C#m','G#m','D#m'],['Dm', 'Am', 'Em' ],['Em', 'Bm', 'F#m'],['F#m','C#m','G#m'],['Gm', 'Dm', 'Am' ],['Am', 'Em', 'Bm' ]
    ]
    
    pProgName = 4  #5th position
    progressions = [
        [6, 4, 1, 5, 'Alternative' ], 
        [1, 5, 6, 4, 'Pop'         ], 
        [1, 6, 3, 7, 'Pop II'      ], 
        [1, 1, 1, 1, 'Didgeridoo'  ],
        [1, 6, 4, 5, 'Dreadful'    ], 
        [1, 6, 2, 5, 'Dreadful II' ], 
        [1, 6, 2, 4, 'Endless'     ],          ### now we can add progressions to this too
        [1, 3, 4, 6, 'Energetic'   ],          
        [1, 4, 3, 6, 'Grunge'      ],           
        [1, 4, 1, 5, 'Memories'    ],         
        [4, 1, 4, 5, 'Rebellious'  ],         
        [1, 4, 5, 5, 'Sad'         ],
        [1, 4, 1, 4, 'Simple'      ], 
        [1, 5, 5, 1, 'Simple II'   ],
        [1, 4, 1, 4, 'Wild Side'   ],
        [1, 1, 4, 6, 'Wistful'     ]  
    ]    
    
    
    KeyChartMajor = [
        [0, 9,23,13, 2, 4,18,32], #0 A
        [0,11,13,15, 4, 6,20,34], #1 B
        [0, 0,14,16, 5, 7,21,35], #2 C
        [0, 2,16,18, 7, 9,23,25], #3 D
        [0, 4,18,20, 9,11,13,27], #4 E
        [0, 5,19,21,10, 0,16,28], #5 F
        [0, 7,21,23, 0, 2,16,30], #6 G
        [0,10,12,14, 3, 5,19,33], #7 A# Cm Dm D# F Gm Adim
        [0, 1,15,17, 6, 8,22,24], #8 C# D#m Fm F# G# A#m Cdim
        [0, 3,17,19, 8,10,12,26], #9 D# Fm Gm G# A# Cm Ddim
        [0, 6,20,22,11, 1,15,29], #10 F# G#m A#m B C# D#m Fdim
        [0, 8,22,12, 1, 3,17,31]  #11 G# A#m Cm C# D# Fm Gdim
    ]
    
    circleFifthsInner = [
        [[0, 5,19,21,10, 0,16,28],[0, 0,14,16, 5, 7,21,35],[0, 7,21,23, 0, 2,16,30]], # Am
        [[0, 7,21,23, 0, 2,16,30],[0, 2,16,18, 7, 9,23,25],[0, 9,23,13, 2, 4,18,32]], # Bm
        [[0, 8,22,12, 1, 3,17,31],[0, 3,17,19, 8,10,12,26],[0,10,12,14, 3, 5,19,33]], # Cm
        [[0,10,12,14, 3, 5,19,33],[0, 5,19,21,10, 0,16,28],[0, 0,14,16, 5, 7,21,35]], # Dm
        [[0, 0,14,16, 5, 7,21,35],[0, 7,21,23, 0, 2,16,30],[0, 2,16,18, 7, 9,23,25]], # Em
        [[0, 1,15,17, 6, 8,22,24],[0, 8,22,12, 1, 3,17,31],[0, 3,17,19, 8,10,12,26]], # Fm
        [[0, 3,17,19, 8,10,12,26],[0,10,12,14, 3, 5,19,33],[0, 5,19,21,10, 0,16,28]], # Gm
        [[0, 5,19,21,10, 0,16,28],[0, 0,14,16, 5, 7,21,35],[0, 7,21,23, 0, 2,16,30]], # Am
        [[0, 8,22,12, 1, 3,17,31],[0, 3,17,19, 8,10,12,26],[0,10,12,14, 3, 5,19,33]], # Cm
        [[0,10,12,14, 3, 5,19,33],[0, 5,19,21,10, 0,16,28],[0, 0,14,16, 5, 7,21,35]], # Dm
        [[0, 1,15,17, 6, 8,22,24],[0, 8,22,12, 1, 3,17,31],[0, 3,17,19, 8,10,12,26]], # Fm
        [[0, 3,17,19, 8,10,12,26],[0,10,12,14, 3, 5,19,33],[0, 5,19,21,10, 0,16,28]], # Gm
    ]
    
    KeyChartMinor = [
        [0,21,35, 0,14,16, 5, 7  ], #0 Am
        [0,23,25, 2,16,18, 7, 9  ], #1 Bm
        [0,12,26, 3,17,19, 8,10  ], #2 Cm
        [0,14,28, 5,19,21,23,10,0], #3 Dm
        [0,16,30, 7,21,23, 0, 2  ], #4 Em
        [0,17,31, 8,22,12, 1, 3  ], #5 Fm
        [0,19,33,10,12,14, 3, 5  ], #6 Gm
        [0,22,24, 1,15,17, 6, 8  ], #7 A#m Cdim C# D#m Fm F# G#
        [0,13,27, 4,18,20, 9,11  ], #8 C#m D#dim E F#m G#m A B
        [0,15,29, 6,20,22,11, 1  ], #9 D#m Fdim F# G#m A#m B C#
        [0,18,32, 9,23,13, 2, 4  ], #10 F#m G#dim A Bm C#m D E
        [0,20,34,11,13,15, 4, 6  ]  #11 G#m A#dim B C#m D#m E F# 
    ]
    
    circleFifthsOuter = [
        [[0,23,25, 2,16,18, 7, 9  ],[0,18,32, 9,23,13, 2, 4  ],[0,13,27, 4,18,20, 9,11]], # A
        [[0,13,27, 4,18,20, 9,11  ],[0,20,34,11,13,15, 4, 6  ],[0,15,29, 6,20,22,11, 1]], # B
        [[0,14,28, 5,19,21,23,10,0],[0,21,35, 0,14,16, 5, 7  ],[0,16,30, 7,21,23, 0, 2]], # C
        [[0,16,30, 7,21,23, 0, 2  ],[0,23,25, 2,16,18, 7, 9  ],[0,18,32, 9,23,13, 2, 4]], # D
        [[0,18,32, 9,23,13, 2, 4  ],[0,13,27, 4,18,20, 9,11  ],[0,20,34,11,13,15, 4, 6]], # E
        [[0,19,33,10,12,14, 3, 5  ],[0,14,28, 5,19,21,23,10,0],[0,21,35, 0,14,16, 5, 7]], # F
        [[0,21,35, 0,14,16, 5, 7  ],[0,16,30, 7,21,23, 0, 2  ],[0,23,25, 2,16,18, 7, 9]], # G
        [[0,23,25, 2,16,18, 7, 9  ],[0,18,32, 9,23,13, 2, 4  ],[0,13,27, 4,18,20, 9,11]], # A
        [[0,14,28, 5,19,21,23,10,0],[0,21,35, 0,14,16, 5, 7  ],[0,16,30, 7,21,23, 0, 2]], # C
        [[0,16,30, 7,21,23, 0, 2  ],[0,23,25, 2,16,18, 7, 9  ],[0,18,32, 9,23,13, 2, 4]], # D
        [[0,19,33,10,12,14, 3, 5  ],[0,14,28, 5,19,21,23,10,0],[0,21,35, 0,14,16, 5, 7]], # F
        [[0,21,35, 0,14,16, 5, 7  ],[0,16,30, 7,21,23, 0, 2  ],[0,23,25, 2,16,18, 7, 9]]  # G
    ]
    
    pChordName = 3 #4th position for name (0,1,2,3)
    Chords = [
        [60,64,67, 'C'    ], #0 C
        [61,65,68, 'C#'   ], #1 C#
        [62,66,69, 'D'    ], #2 D
        [63,67,70, 'D#'   ], #3 D#
        [64,68,71, 'E'    ], #4 E
        [65,69,72, 'F'    ], #5 F
        [66,70,73, 'F#'   ], #6 F#
        [67,71,74, 'G'    ], #7 G
        [68,72,75, 'G#'   ], #8 G#
        [69,73,76, 'A'    ], #9 A
        [70,74,77, 'A#'   ], #10 A#
        [71,75,78, 'B'    ], #11 B
        [60,63,67, 'Cm'   ], #12 Cmin
        [61,64,68, 'C#m'  ], #13 C#min
        [62,65,69, 'Dm'   ], #14 Dmin
        [63,66,70, 'D#m'  ], #15 D#min
        [64,67,71, 'Em'   ], #16 Emin
        [65,68,72, 'Fm'   ], #17 Fmin
        [66,69,73, 'F#m'  ], #18 F#min
        [67,70,74, 'Gm'   ], #19 Gmin
        [68,71,75, 'G#m'  ], #20 G#min
        [69,72,76, 'Am'   ], #21 Amin
        [70,73,77, 'A#m'  ], #22 A#min
        [71,74,78, 'Bm'   ], #23 Bmin
        [60,63,66, 'Cdim' ], #24 Cdim
        [61,64,67, 'C#dim'], #25 C#dim
        [62,65,68, 'Ddim' ], #26 Ddim
        [63,66,69, 'D#dim'], #27 D#dim
        [64,68,70, 'Edim' ], #28 Edim
        [65,68,71, 'Fdim' ], #29 Fdim
        [66,69,72, 'F#dim'], #30 F#dim
        [67,70,73, 'Gdim' ], #31 Gdim
        [68,71,74, 'G#dim'], #32 G#dim
        [69,72,75, 'Adim' ], #33 Adim
        [70,73,76, 'A#dim'], #34 A#dim
        [71,74,77, 'Bdim' ]  #35 Bdim
    ]
    
    ############### end of static data ##################
    
    def __init__(self):
        #init stuff here
        self.rt = RegionToolbox()  #small class at end of this file
        
        self.quartNoteLength = 960 #quarter note length in ticks
        p, bpm, bpi = RPR_GetProjectTimeSignature2(0, 0, 0)
        bps = 60/bpm
        self.barLength = bps * bpi
        self.barsPerSection = 4  #TODO doesn't work unless 4 at the mo
        
        for i, val in enumerate(RC.progressions):
            RC.Moods+=(val[RC.pProgName],)                      ### this section populates the tuple accessed by 
                                                                ### the combo box creation code with names from new combined lists
        for i, val in enumerate(RC.Structures):                 
            RC.entriesStructures+=(val[RC.pStructureName],)
            
        self.RetrieveSettings()

        
    
    def StoreSettings(self):
        self.msg("StoreSettings - enter")
        for key, lst in RC.Settings.items():
            RPR_SetExtState(RC.REACHORD, str(key), str(lst[0]), True)
            RPR_SetExtState(RC.REACHORD, str(key)+"1", str(lst[1]), True)
            self.msg("Storing... "+key+".... "+str(lst[0]+"  ....  "+str(lst[1])))
        
        
    def RetrieveSettings(self):
        self.msg("RetrieveSettings - enter")
        if RPR_GetExtState(RC.REACHORD, "FirstTime") is not "NotFirstTime":
            for key, val in RC.Settings.items():
                RC.Settings[key][0] = str(RPR_GetExtState(RC.REACHORD, str(key)))
                RC.Settings[key][1] = int(RPR_GetExtState(RC.REACHORD, str(key)+"1"))
                self.msg("Retrieved... "+key+".... "+str(RC.Settings[key][0])+".... "+str(RC.Settings[key][1]))
            else:
                RPR_SetExtState(RC.REACHORD, "FirstTime", "NotFirstTime", True)
        self.msg("RetrieveSettings - exit")
    
    
    def drawChordText(self,mood,inKey,scale,structure):
        self.msg('drawChordText - Enter')
    
        rand = randint(0, 2)
        structure = int(structure)
        
        self.msg('--- check scale')
        if scale == 'Major':
             chartselect = RC.KeyChartMajor[int(inKey)-1]
             useCircle = RC.circleFifthsOuter[int(inKey)-1][int(rand)]
        else: #minor
             chartselect = RC.KeyChartMinor[int(inKey)-1]
             useCircle = RC.circleFifthsInner[int(inKey)-1][int(rand)]
        
        self.msg('--- get verse chords')   
        verse = '\n-----------------------------------------------------------\n'
        verse += 'Verse:  '
        for i, val in enumerate(RC.progressions[int(mood)-1]):
            if i == RC.pProgName : break
            x = int(chartselect[int(val)])
            verse += str(RC.Chords[x][RC.pChordName]) + '  '
             
        chorus = '\n-----------------------------------------------------------\n'
        chorus += 'Chorus: '
        for i, val in enumerate(RC.progressions[int(mood)-1]):
            if i == RC.pProgName : break
            x = int(useCircle[int(val)])
            chorus += str(RC.Chords[x][RC.pChordName]) + '  '        
            data=""
            
            if structure == 0:
                data = verse
            else:
                self.msg("Structure = "+ RC.Structures[structure-1][RC.pStructureChars])
                for i, val in enumerate(RC.Structures[structure-1][RC.pStructureChars]):
                    #self.msg("Text... part = " + str(val))
                    if val == 'V':
                        data+=verse
                    else:
                        data+=chorus
        
        self.msg('drawChordText - Exit')  
        return 'Proposed chord progression for a song' + data
    
    def msg(self,m):
        global msg
        msg(m)
        #self.console.config(state="normal") #set read-write
        #self.console.insert(Tkinter.INSERT, str(m))
        #self.console.config(state="disabled") #set read-write
        
    
    def drawChordTextKey(self,mood,inKey,scale,structure):
        self.msg('rc.drawChordTextKey')
        if scale == 'Major':
            self.msg('--- is major')
            chartselect = RC.KeyChartMajor[int(inKey)-1]
            ret = ''
            for i, val in enumerate(chartselect):
                if i >=1:
                    self.msg('--- i>=1')
                    ret += str(RC.Chords[int(val)][RC.pChordName]) + ' '
            self.msg('DrawChordTextKey - Exit')
            return ret
        elif scale == 'Minor': #if minor
            self.msg('--- is minor')
            chartselect = RC.KeyChartMinor[int(inKey)-1]
            ret = ''
            for i, val in enumerate(chartselect):
                if i >=1:
                    ret += str(RC.Chords[int(val)][RC.pChordName]) + ' '
            self.msg('DrawChordTextKey - Exit')
            return ret
        self.msg('-- exit, not major or minor')
        
            
    def cmdME(self,actME, cmd):
        RPR_MIDIEditor_OnCommand(actME,cmd)
   
          
    def drawMidi(self,mood,inKey,scale,structure,layout):
        self.msg('RC.drawMidi - Enter')
        p, bpm, bpi = RPR_GetProjectTimeSignature2(0, 0, 0)
        bps = 60/bpm
        barLength = bps * bpi
        self.beatsInBar = bpi
        self.msg("wtf")
        self.selAllNotesTselection()
        self.deleteSelectedNotes()
        self.msg("wtf2")
        sectionLength = self.quartNoteLength * self.beatsInBar * self.barsPerSection #assumes self.barsPerSection bars per sections
        self.msg(sectionLength)
        
        if RC.Settings[RC.REGION][RC.pSettingsOnOff] == RC.TRUE:
            self.rt.DeleteAllRegions()
        currentPos = 0
        
        #################
        
        velocity = 96
        selected = 1
        self.chordLength = self.quartNoteLength* self.beatsInBar
        randNote = randint(2, 3)
        randPos = [0,480,960]
        randLength = [0.5, 1, 2]
        x=chartselect=useCircle=startPosition=position=0
        self.msg('--- About to draw parts')   
        ii=0
        #if not (all(c in RC.allowedChars for c in RC.Structures[structure-1])) :
        #    structure+=1   #make sure it's not the choose option in the drop down
        rand = randint(0, 2)
        for i, part in enumerate(RC.Structures[structure-1][RC.pStructureChars]):
            self.msg('---------------Iterate through structure...' + str(ii) + '------------------')
            self.msg(RC.Structures[structure-1][RC.pStructureChars])   
            msg('Size of progression..' + str(len(RC.progressions[int(mood)-1])))
            for c, chord in enumerate(RC.progressions[int(mood)-1]):
                if c is RC.pProgName : break
                msg("*******************" + "GOING THROUGH PROGRESSION" + "*******************")
                msg("******************* CHORD # " + str(c) + " **************************************")
                msg("******************* IN KEY # " + str(inKey) + "***************************************")
                msg("******************* PART IS " +str(part) + "********************************************")
                if scale == 'Major':
                    chartselect = RC.KeyChartMajor[int(inKey)-1]
                    useCircle = RC.circleFifthsOuter[int(inKey)-1][int(rand)]
                    self.msg('--- is major')
                else: #default to minor
                    chartselect = RC.KeyChartMinor[int(inKey)-1]
                    useCircle = RC.circleFifthsInner[int(inKey)-1][int(rand)]
                    self.msg('--- is minor')
                #if i == self.beatsInBar - 1 : break #if time sig less than 4, only do 3 (probably be better picking random ones)
                if part == 'V':
                    x = int(chartselect[int(chord)])
                    self.msg('--- is verse - chart = ' + str(x))
                elif part == 'C': #is chorus
                    x = int(useCircle[int(chord)])
                    self.msg('--- is chorus - circle = ' + str(x))
        
            
                msg(RC.Settings[RC.CHORDS][RC.pSettingsOnOff])
                msg(RC.TRUE)
                if RC.Settings[RC.CHORDS][RC.pSettingsOnOff] == RC.TRUE:    # need to draw chords
                    self.msg('about to draw chords')
                    channel = int(RC.Settings[RC.CHORDS][RC.pSettingsChannel])
                    self.msg('-- channel set' + str(channel))
                    self.drawChords(channel,velocity,x,currentPos,startPosition,rand)
                if RC.Settings[RC.MELODY][RC.pSettingsOnOff] == RC.TRUE:    # need to draw melody
                    self.msg('about to draw melody')
                    self.drawMelody(int(RC.Settings[RC.MELODY][RC.pSettingsChannel]),velocity,x,currentPos,startPosition,rand, randNote)
                if RC.Settings[RC.BASS][RC.pSettingsOnOff] == RC.TRUE:
                    self.drawBass(RC.Settings[RC.BASS][RC.pSettingsChannel],velocity,x,currentPos,startPosition,rand)
                    
                currentPos += sectionLength/self.beatsInBar
            
            if RC.Settings[RC.REGION][RC.pSettingsOnOff] == RC.TRUE:
                self.msg('Region start pos: ' + str(self.barsPerSection*ii))
                self.rt.AddRegion(barLength,self.barsPerSection*ii, self.barsPerSection, str(part), 0)
            ii+=1
        
        self.msg('drawMIDI - Exit')

 
    def drawMelody(self,channel,velocity,x,position,startPosition,rd, randNote):
        self.msg('drawMelody - Enter')
        take = self.getActTakeInEditor()
        midiTake = self.allocateMIDITake(take)
        self.msg('--- take allocated:' + str(midiTake))
        randNote = randint(2, 3)
        randPos = [0,480,960]
        randLength = [0.5, 1, 2]
        
        for j, vm in enumerate(RC.Chords[x]):
            if j == RC.pChordName or j == self.beatsInBar - 1 : break #don't parse chordname and do 3 notes per bar for 3/4
            if j % randNote == 0:
               self.msg('--- draw note, random length 0.5, 1, 2')
               rpos = randPos[randint(0,2)]
               rlen = randLength[randint(0,2)]
               self.msg('--- rpos rlen set')
               note = self.addNote(midiTake, int(RC.Settings[RC.MELODY][RC.pSettingsChannel]), int(velocity), int(position + rpos), int(RC.Chords[x][randint(0, 2)]),\
                                   int(self.quartNoteLength * rlen), 0)
               position += rpos + (self.quartNoteLength * rlen)
               #note = self.addNote(midiTake, RC.Settings[RC.MELODY][RC.pSettingsChannel], velocity, position, int(vm), self.quartNoteLength, selected)
            else:
               self.msg('--- else draw note')
               note = self.addNote(midiTake, int(RC.Settings[RC.MELODY][RC.pSettingsChannel]), int(velocity), int(position), int(vm), int(self.quartNoteLength), 0)
               position += self.quartNoteLength
        self.freeMIDITake(midiTake)
        self.msg('drawMelody - Exit')


    def drawChords(self,channel,velocity,x,position,startPosition,rd):        
        self.msg('drawChords - Enter')
        take = self.getActTakeInEditor()
        midiTake = self.allocateMIDITake(take)
        self.msg('--- take allocated:' + str(midiTake))

        note = self.addNote(midiTake,int(RC.Settings[RC.CHORDS][RC.pSettingsChannel]),int(velocity),int(position + startPosition),\
                            int(RC.Chords[x][0])-12,int(self.chordLength),0)
        note = self.addNote(midiTake,int(RC.Settings[RC.CHORDS][RC.pSettingsChannel]),int(velocity),int(position + startPosition),\
                            int(RC.Chords[x][1])-12,int(self.chordLength),0)
        note = self.addNote(midiTake,int(RC.Settings[RC.CHORDS][RC.pSettingsChannel]),int(velocity),int(position + startPosition),\
                            int(RC.Chords[x][2])-12,int(self.chordLength),0)
        position += self.chordLength
        self.freeMIDITake(midiTake)
        self.msg('DrawChords - Exit')
     
        
    def drawBass(self,channel,velocity,x,position,startPosition,rd):
        self.msg('drawBass - Enter')
        take = self.getActTakeInEditor()
        midiTake = self.allocateMIDITake(take)
        self.msg('--- take allocated:' + str(midiTake))
        self.msg('drawBass - Enter')
        note = self.addNote(midiTake,int(RC.Settings[RC.BASS][RC.pSettingsChannel]),int(velocity),int(position + startPosition),\
                            int(RC.Chords[x][0])-24,int(self.chordLength),0)
        self.freeMIDITake(midiTake)
        self.msg('drawBass - Exit')
      
    def drawScale(self,channel,velocity,key,position,startPosition):
        self.msg('drawScale - Enter')

    def getActTakeInEditor(self):
        return RPR_MIDIEditor_GetTake(RPR_MIDIEditor_GetActive())
    
    def allocateMIDITake(self,midiTake):
        return FNG_AllocMidiTake(midiTake)
    
    def freeMIDITake(self,midiTake):
        FNG_FreeMidiTake(midiTake)
    
    def countNotes(self,midiTake):
        return FNG_CountMidiNotes(midiTake)
    
    def getMidiNote(self,midiTake, index):
        return FNG_GetMidiNote(midiTake, index)
    
    def getMidiNoteIntProperty(self,midiNote, prop):
        return FNG_GetMidiNoteIntProperty(midiNote, prop)
    
    def setMidiNoteIntProperty(self,midiNote, prop, value):
        FNG_SetMidiNoteIntProperty(midiNote, prop, value)
        
    def selAllNotesTselection(self,command_id=40746, islistviewcommand=0):
        RPR_MIDIEditor_LastFocused_OnCommand(command_id, islistviewcommand)
    
    def deleteSelectedNotes(self,command_id=40002, islistviewcommand=0):
        RPR_MIDIEditor_LastFocused_OnCommand(command_id, islistviewcommand)
        
    def addNote(self,midiTake, ch, vel, pos, pitch, length, sel):
        self.msg('addNote - Enter')
        midiNote = FNG_AddMidiNote(midiTake)
        FNG_SetMidiNoteIntProperty(midiNote, "CHANNEL", ch)
        FNG_SetMidiNoteIntProperty(midiNote, "VELOCITY", vel)
        FNG_SetMidiNoteIntProperty(midiNote, "POSITION", pos)
        FNG_SetMidiNoteIntProperty(midiNote, "PITCH", pitch)
        FNG_SetMidiNoteIntProperty(midiNote, "LENGTH", length)
        FNG_SetMidiNoteIntProperty(midiNote, "SELECTED", sel)
        return midiNote  
    
    
class RegionToolbox:
    """
    -------------------------------------------------------------------------------------------------------------
    
    -------------------------------------------------------------------------------------------------------------
    """
    def AddRegion(self, barLength, startBar, lengthBars, name, colour):
        global msg
        msg("AddRegion - Enter")
       
        return RPR_AddProjectMarker(0, 1, startBar*barLength, (startBar*barLength)+(lengthBars*barLength), name, 1)  #colour doesn't work for some reason     
      
   
    def DeleteAllRegions(self):
        #thanks to spk77 for this
        i = 0
        regionIndexNumL = []  

        while RPR_EnumProjectMarkers(i, 1, 0, 0, "", 0)[0]:
           a, index, isRegion, start, end, name, markerIndex = RPR_EnumProjectMarkers(i, 1, 0, 0, "", 0)
           if isRegion:
              regionIndexNumL.append(markerIndex)
           i += 1
 
        for region in regionIndexNumL:
           RPR_DeleteProjectMarker(0, int(region), 1)
         
         
        